@@ -15,7 +15,7 @@
  *                                                                         *
  ***************************************************************************/
 
-#define VERSION "1.0.18"
+#define VERSION "1.0.32"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -31,10 +31,6 @@
 #include <windows.h>
 #endif
 
-#ifdef WIN32
-#define strcasecmp stricmp
-#endif
-
 #include "xchat-plugin.h"
 #include "tclplugin.h"
 #include "printevents.h"
@@ -175,33 +171,61 @@ static char *myitoa(int value)
     return result;
 }
 
+
 static xchat_context *xchat_smart_context(char *arg1, char *arg2)
 {
-    /* slightly tweak xchat_find_context's amazing psychic abilities */
+    char *server, *s, *n;
+    xchat_context *result = NULL;
+    xchat_context *ctx = NULL;
+    xchat_list *list;
 
-    char *server = NULL;
-    char *channel = NULL;
+    if (arg1 == NULL)
+        return xchat_get_context(ph);;
 
-    if (arg1 && !arg2) {
-        if (xchat_find_context(ph, arg1, arg1))
-            server = arg1;
-        else
-            channel = arg1;
-    } else if (arg1 && arg2) {
-        server = arg1;
-        channel = arg2;
-    } else
-        return NULL;
+    if (arg1 && arg2) {
+        result = xchat_find_context(ph, arg1, arg2);
+        if (result == NULL)
+            result = xchat_find_context(ph, arg2, arg1);
+        return result;
+    } else {
 
-    if (server && channel)
-        return xchat_find_context(ph, server, channel);
-    else if (server && !channel)
-        return xchat_find_context(ph, server, server);
-    else if (!server && channel)
-        return xchat_find_context(ph, (char *) xchat_get_info(ph, "server"), channel);
-    else
-        return NULL;
+        server = xchat_get_info(ph, "server");
 
+        list = xchat_list_get(ph, "channels");
+
+        if (list != NULL) {
+
+            while (xchat_list_next(ph, list)) {
+
+                s = xchat_list_str(ph, list, "server");
+                ctx = xchat_list_str(ph, list, "context");
+
+                if (xchat_list_int(ph, list, "type") == 1) {
+                    if (strcasecmp(arg1, s) == 0) {
+                        result = ctx;
+                        break;
+                    }
+                    n = xchat_list_str(ph, list, "network");
+                    if (n) {
+                        if (strcasecmp(arg1, n) == 0) {
+                            result = ctx;
+                            break;
+                        }
+                    }
+                } else {
+                    if ((strcasecmp(server, s) == 0) && (strcasecmp(arg1, xchat_list_str(ph, list, "channel")) == 0)) {
+                        result = ctx;
+                        break;
+                    }
+                }
+            }
+
+            xchat_list_free(ph, list);
+        }
+
+    }
+
+    return result;
 }
 
 static void queue_nexttimer()
@@ -240,9 +264,9 @@ static int insert_timer(int seconds, char *script)
     for (x = 1; x < MAX_TIMERS; x++) {
         if (timers[x].timerid == 0) {
             if (SourceInternalProc(id, "", script) == TCL_ERROR) {
-               errorInfo = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
-               xchat_printf(ph, "\0039Tcl plugin\003\tERROR (timer %d) %s \n", timers[x].timerid, errorInfo);
-               return (-1);
+                errorInfo = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
+                xchat_printf(ph, "\0039TCL plugin\003\tERROR (timer %d) %s \n", timers[x].timerid, errorInfo);
+                return (-1);
             }
             timers[x].timerid = (nexttimerid++ % INT_MAX) + 1;
             timers[x].timestamp = now + seconds;
@@ -274,7 +298,7 @@ static void do_timer()
     origctx = xchat_get_context(ph);
     if (EvalInternalProc(timers[index].procPtr, 0) == TCL_ERROR) {
         errorInfo = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
-        xchat_printf(ph, "\0039Tcl plugin\003\tERROR (timer) %s \n", errorInfo);
+        xchat_printf(ph, "\0039TCL plugin\003\tERROR (timer) %s \n", errorInfo);
     }
     xchat_set_context(ph, origctx);
 
@@ -401,7 +425,7 @@ static int Server_raw_line(char *word[], char *word_eol[], void *userdata)
                 origctx = xchat_get_context(ph);
                 if (EvalInternalProc(proc_argv[1], 7, src, dest, cmd, rest, word_eol[1], proc_argv[0], myitoa(private)) == TCL_ERROR) {
                     errorInfo = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
-                    xchat_printf(ph, "\0039Tcl plugin\003\tERROR (on %s %s) %s \n", cmd, proc_argv[0], errorInfo);
+                    xchat_printf(ph, "\0039TCL plugin\003\tERROR (on %s %s) %s \n", cmd, proc_argv[0], errorInfo);
                 }
                 xchat_set_context(ph, origctx);
 
@@ -467,7 +491,7 @@ static int Print_Hook(char *word[], void *userdata)
                     Tcl_DStringAppend(&ds, word[1], strlen(word[1]));
                     if (EvalInternalProc(proc_argv[1], 7, ds.string, word[2], xc[(int) userdata].event, word[4], "", proc_argv[0], "0") == TCL_ERROR) {
                         errorInfo = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
-                        xchat_printf(ph, "\0039Tcl plugin\003\tERROR (on %s %s) %s \n", xc[(int) userdata].event, proc_argv[0], errorInfo);
+                        xchat_printf(ph, "\0039TCL plugin\003\tERROR (on %s %s) %s \n", xc[(int) userdata].event, proc_argv[0], errorInfo);
                     }
                 } else {
                     if (xc[(int) userdata].argc > 0) {
@@ -839,7 +863,7 @@ static int tcl_command(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
     xchat_context *ctx = NULL;
     char *string = NULL;
 
-    BADARGS(2, 4, " ?server? ?#channel|nick? text");
+    BADARGS(2, 4, " ?server|network? ?#channel|nick? text");
 
     origctx = xchat_get_context(ph);
 
@@ -876,7 +900,7 @@ static int tcl_raw(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
     xchat_context *ctx = NULL;
     char *string = NULL;
 
-    BADARGS(2, 4, " ?server? ?#channel|nick? text");
+    BADARGS(2, 4, " ?server|network? ?#channel|nick? text");
 
     origctx = xchat_get_context(ph);
 
@@ -949,7 +973,10 @@ static int tcl_getlist(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
     int iattr;
     int i;
     Tcl_DString ds;
-    xchat_context *ctx;
+    xchat_context *origctx;
+    xchat_context *ctx = NULL;
+
+    origctx = xchat_get_context(ph);
 
     BADARGS(1, 2, " list");
 
@@ -1008,6 +1035,7 @@ static int tcl_getlist(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
                 sattr = xchat_list_str(ph, list, (char *) field);
                 if (strcmp(field, "context") == 0) {
                     ctx = (xchat_context *) sattr;
+                    xchat_set_context(ph, ctx);
                     Tcl_DStringStartSublist(&ds);
                     Tcl_DStringAppendElement(&ds, "ctx");
                     Tcl_DStringAppendElement(&ds, xchat_get_info(ph, "server"));
@@ -1030,6 +1058,8 @@ static int tcl_getlist(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 
   done:
 
+    xchat_set_context(ph, origctx);
+
     Tcl_AppendResult(irp, ds.string, NULL);
 
     Tcl_DStringFree(&ds);
@@ -1136,7 +1166,7 @@ static int tcl_print(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
     xchat_context *ctx = NULL;
     char *string = NULL;
 
-    BADARGS(2, 4, " ?server? ?#channel|nick? text");
+    BADARGS(2, 4, " ?server|network? ?#channel|nick? text");
 
     origctx = xchat_get_context(ph);
 
@@ -1192,7 +1222,7 @@ static int tcl_findcontext(ClientData cd, Tcl_Interp * irp, int argc, char *argv
     xchat_context *origctx, *ctx = NULL;
     Tcl_DString ds;
 
-    BADARGS(1, 3, " ?server? ?channel?");
+    BADARGS(1, 3, " ?server|network? ?channel?");
 
     switch (argc) {
     case 1:
@@ -1225,7 +1255,6 @@ static int tcl_findcontext(ClientData cd, Tcl_Interp * irp, int argc, char *argv
     return TCL_OK;
 }
 
-
 static int tcl_getcontext(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 {
     xchat_context *ctx = NULL;
@@ -1253,13 +1282,20 @@ static int tcl_channels(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
     char *server, *channel;
     xchat_list *list;
     Tcl_DString ds;
+    xchat_context *origctx;
+    xchat_context *ctx;
 
-    BADARGS(1, 2, " ?server?");
+    origctx = xchat_get_context(ph);
 
-    if (argc == 1)
-        server = (char *) xchat_get_info(ph, "server");
-    else
-        server = argv[1];
+    BADARGS(1, 2, " ?server|network?");
+
+    if (argc == 2) {
+        ctx = xchat_find_context(ph, argv[1], NULL);
+        CHECKCTX(ctx);
+        xchat_set_context(ph, ctx);
+    }
+
+    server = (char *) xchat_get_info(ph, "server");
 
     Tcl_DStringInit(&ds);
 
@@ -1267,25 +1303,29 @@ static int tcl_channels(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 
     if (list != NULL) {
         while (xchat_list_next(ph, list)) {
-            if (strcmp(server, xchat_list_str(ph, list, "server")) != 0)
+            if (xchat_list_int(ph, list, "type") != 2)
+                continue;
+            if (strcasecmp(server, xchat_list_str(ph, list, "server")) != 0)
                 continue;
             (const char *) channel = xchat_list_str(ph, list, "channel");
-            if ((channel[0] == '#') || (channel[0] == '&'))
-                Tcl_DStringAppendElement(&ds, channel);
+            Tcl_DStringAppendElement(&ds, channel);
         }
         xchat_list_free(ph, list);
     }
 
+
     Tcl_AppendResult(irp, ds.string, NULL);
 
     Tcl_DStringFree(&ds);
 
+    xchat_set_context(ph, origctx);
+
     return TCL_OK;
 }
 
 static int tcl_servers(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 {
-    char *server, *channel;
+    char *server;
     xchat_list *list;
     Tcl_DString ds;
 
@@ -1297,10 +1337,10 @@ static int tcl_servers(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 
     if (list != NULL) {
         while (xchat_list_next(ph, list)) {
-            (const char *) channel = xchat_list_str(ph, list, "channel");
-            (const char *) server = xchat_list_str(ph, list, "server");
-            if (strcmp(channel, server) == 0)
+            if (xchat_list_int(ph, list, "type") == 1) {
+                (const char *) server = xchat_list_str(ph, list, "server");
                 Tcl_DStringAppendElement(&ds, server);
+            }
         }
         xchat_list_free(ph, list);
     }
@@ -1317,13 +1357,20 @@ static int tcl_queries(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
     char *server, *channel;
     xchat_list *list;
     Tcl_DString ds;
+    xchat_context *origctx;
+    xchat_context *ctx;
 
-    BADARGS(1, 2, " ?server?");
+    origctx = xchat_get_context(ph);
 
-    if (argc == 1)
-        server = (char *) xchat_get_info(ph, "server");
-    else
-        server = argv[1];
+    BADARGS(1, 2, " ?server|network?");
+
+    if (argc == 2) {
+        ctx = xchat_find_context(ph, argv[1], NULL);
+        CHECKCTX(ctx);
+        xchat_set_context(ph, ctx);
+    }
+
+    server = (char *) xchat_get_info(ph, "server");
 
     Tcl_DStringInit(&ds);
 
@@ -1331,13 +1378,12 @@ static int tcl_queries(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 
     if (list != NULL) {
         while (xchat_list_next(ph, list)) {
-            if (strcmp(server, xchat_list_str(ph, list, "server")) != 0)
+            if (xchat_list_int(ph, list, "type") != 3)
+                continue;
+            if (strcasecmp(server, xchat_list_str(ph, list, "server")) != 0)
                 continue;
             (const char *) channel = xchat_list_str(ph, list, "channel");
-            if (strcmp(channel, server) == 0)
-                continue;
-            if ((channel[0] != '#') && (channel[0] != '&'))
-                Tcl_DStringAppendElement(&ds, channel);
+            Tcl_DStringAppendElement(&ds, channel);
         }
         xchat_list_free(ph, list);
     }
@@ -1346,6 +1392,8 @@ static int tcl_queries(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 
     Tcl_DStringFree(&ds);
 
+    xchat_set_context(ph, origctx);
+
     return TCL_OK;
 }
 
@@ -1355,7 +1403,7 @@ static int tcl_users(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
     xchat_list *list;
     Tcl_DString ds;
 
-    BADARGS(1, 3, " ?server? ?channel?");
+    BADARGS(1, 3, " ?server|network? ?channel?");
 
     origctx = xchat_get_context(ph);
 
@@ -1388,7 +1436,7 @@ static int tcl_users(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
         Tcl_DStringAppendElement(&ds, "prefix");
         Tcl_DStringEndSublist(&ds);
 
-        while (xchat_list_next(ph, list)) {;
+        while (xchat_list_next(ph, list)) {
             Tcl_DStringStartSublist(&ds);
             Tcl_DStringAppendElement(&ds, (const char *) xchat_list_str(ph, list, "nick"));
             Tcl_DStringAppendElement(&ds, (const char *) xchat_list_str(ph, list, "host"));
@@ -1420,7 +1468,7 @@ static int tcl_chats(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
     list = xchat_list_get(ph, "dcc");
 
     if (list != NULL) {
-        while (xchat_list_next(ph, list)) {;
+        while (xchat_list_next(ph, list)) {
             switch (xchat_list_int(ph, list, "type")) {
             case 2:
             case 3:
@@ -1453,7 +1501,7 @@ static int tcl_ignores(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 
     if (list != NULL) {
 
-        while (xchat_list_next(ph, list)) {;
+        while (xchat_list_next(ph, list)) {
             Tcl_DStringStartSublist(&ds);
             Tcl_DStringAppendElement(&ds, (const char *) xchat_list_str(ph, list, "mask"));
             Tcl_DStringStartSublist(&ds);
@@ -1499,7 +1547,7 @@ static int tcl_dcclist(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 
     if (list != NULL) {
 
-        while (xchat_list_next(ph, list)) {;
+        while (xchat_list_next(ph, list)) {
             Tcl_DStringStartSublist(&ds);
             dcctype = xchat_list_int(ph, list, "type");
             switch (dcctype) {
@@ -1569,7 +1617,7 @@ static int tcl_dcclist(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 static int tcl_away(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 {
     xchat_context *origctx, *ctx;
-    BADARGS(1, 2, " ?server?");
+    BADARGS(1, 2, " ?server|network?");
 
     origctx = xchat_get_context(ph);
 
@@ -1589,7 +1637,7 @@ static int tcl_away(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 static int tcl_host(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 {
     xchat_context *origctx, *ctx;
-    BADARGS(1, 2, " ?server?");
+    BADARGS(1, 2, " ?server|network?");
 
     origctx = xchat_get_context(ph);
 
@@ -1609,7 +1657,7 @@ static int tcl_host(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 static int tcl_network(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 {
     xchat_context *origctx, *ctx;
-    BADARGS(1, 2, " ?server?");
+    BADARGS(1, 2, " ?server|network?");
 
     origctx = xchat_get_context(ph);
 
@@ -1629,7 +1677,7 @@ static int tcl_network(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 static int tcl_me(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 {
     xchat_context *origctx, *ctx;
-    BADARGS(1, 2, " ?server?");
+    BADARGS(1, 2, " ?server|network?");
 
     origctx = xchat_get_context(ph);
 
@@ -1649,7 +1697,7 @@ static int tcl_me(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 static int tcl_topic(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 {
     xchat_context *origctx, *ctx = NULL;
-    BADARGS(1, 3, " ?server? ?channel?");
+    BADARGS(1, 3, " ?server|network? ?channel?");
 
     origctx = xchat_get_context(ph);
 
@@ -1843,8 +1891,9 @@ static void Tcl_Plugin_Init()
     char *xchatdir;
 
     interp = Tcl_CreateInterp();
+
     Tcl_Init(interp);
-    
+
     nextprocid = 0x1000;
 
     Tcl_CreateCommand(interp, "alias", tcl_alias, NULL, NULL);
@@ -1965,27 +2014,18 @@ static void banner()
 int xchat_plugin_init(xchat_plugin * plugin_handle, char **plugin_name, char **plugin_desc, char **plugin_version, char *arg)
 {
 #ifdef WIN32
-    static int have_lib = 0;
     HINSTANCE lib;
 #endif
 
     ph = plugin_handle;
 
 #ifdef WIN32
-    if (!have_lib) {
-        lib = LoadLibrary("tcl84.dll");
-        if (!lib) {
-            banner();
-            xchat_print(ph, "---\t---\n");
-            xchat_print(ph, "Cannot open tcl84.dll.\n");
-            xchat_print(ph, "You must have ActiveTcl 8.4 installed in order to run Tcl scripts.\n");
-            xchat_print(ph, "http://www.activestate.com/ActiveTcl/\n");
-            xchat_print(ph, "Make sure Tcl's bin directory is in your PATH.\n");
-            return 0;
-        }
-        have_lib++;
-        FreeLibrary(lib);
+    lib = LoadLibraryA(TCL_DLL);
+    if (!lib) {
+        xchat_print(ph, "You must have ActiveTCL installed in order to run Tcl scripts.\n" "http://aspn.activestate.com/ASPN/Tcl/\n" "Make sure Tcl's bin directory is in your PATH.\n\n");
+        return 0;
     }
+    FreeLibrary(lib);
 #endif
 
     if (initialized != 0) {
@@ -1997,7 +2037,7 @@ int xchat_plugin_init(xchat_plugin * plugin_handle, char **plugin_name, char **p
     initialized = 1;
 
     *plugin_name = "Tcl plugin";
-    *plugin_desc = "TCL scripting interface";
+    *plugin_desc = "Tcl scripting interface";
     *plugin_version = VERSION;
 
     Tcl_Plugin_Init();
