@@ -91,6 +91,21 @@ dcc_unthrottle (struct DCC *dcc)
 		dcc_send_data (NULL, 0, dcc);
 }
 
+#ifdef WIN32
+#include <windows.h>
+
+/* g_get_current_time is giving bad CPS results, let's try this instead */
+static void
+g_get_current_time_win32 (GTimeVal *tv)
+{
+	SYSTEMTIME current;
+
+	GetLocalTime (&current);
+	tv->tv_sec = current.wSecond;
+	tv->tv_usec = current.wMilliseconds * 1000;
+}
+#endif
+
 static void
 dcc_calc_cps (struct DCC *dcc)
 {
@@ -100,7 +115,11 @@ dcc_calc_cps (struct DCC *dcc)
 	int glob_throttle_bit, wasthrottled;
 	int *cpssum, glob_limit;
 
+#ifdef WIN32
+	g_get_current_time_win32 (&now);
+#else
 	g_get_current_time (&now);
+#endif
 
 	/* the pos we use for sends is an average
 		between pos and ack */
@@ -1462,7 +1481,7 @@ dcc_resume (struct DCC *dcc)
 }
 
 void
-handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
+handle_dcc (struct session *sess, char *nick, char *word[],
 				char *word_eol[])
 {
 	char tbuf[512];
@@ -1538,9 +1557,9 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 							file_part (dcc->file), port, dcc->resumable);
 				dcc->serv->p_ctcp (dcc->serv, dcc->nick, tbuf);
 			}
-			sprintf (outbuf, "%u", dcc->pos);
+			sprintf (tbuf, "%u", dcc->pos);
 			EMIT_SIGNAL (XP_TE_DCCRESUMEREQUEST, sess, nick,
-							 file_part (dcc->file), outbuf, NULL, 0);
+							 file_part (dcc->file), tbuf, NULL, 0);
 		}
 		return;
 	}
@@ -1654,10 +1673,10 @@ dontresume:
 			else
 				fe_dcc_update_recv_win ();
 		}
-		sprintf (outbuf, "%lu", size);
-		snprintf (outbuf + 24, 300, "%s:%d", net_ip (dcc->addr), dcc->port);
+		sprintf (tbuf, "%lu", size);
+		snprintf (tbuf + 24, 300, "%s:%d", net_ip (dcc->addr), dcc->port);
 		EMIT_SIGNAL (XP_TE_DCCSENDOFFER, sess->server->front_session, nick,
-						 file, outbuf, outbuf + 24, 0);
+						 file, tbuf, tbuf + 24, 0);
 	} else
 		EMIT_SIGNAL (XP_TE_DCCGENERICOFFER, sess->server->front_session,
 						 word_eol[4] + 2, nick, NULL, NULL, 0);
