@@ -50,6 +50,7 @@
 #include "xchatc.h"
 #include "servlist.h"
 #include "server.h"
+#include "tree.h"
 #include "outbound.h"
 
 
@@ -855,92 +856,92 @@ cmd_deop (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	}
 }
 
+typedef struct
+{
+	char **nicks;
+	int i;
+	session *sess;
+	char *reason;
+	char *tbuf;
+} multidata;
+
+static int
+mdehop_cb (struct User *user, multidata *data)
+{
+	if (user->hop && !user->me)
+	{
+		data->nicks[data->i] = user->nick;
+		data->i++;
+	}
+	return TRUE;
+}
+
 static int
 cmd_mdehop (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
-	struct User *user;
 	char **nicks = malloc (sizeof (char *) * sess->hops);
-	int i = 0;
-	GSList *list = sess->userlist;
-
-	while (list)
-	{
-		user = (struct User *) list->data;
-		if (user->hop && (strcmp (user->nick, sess->server->nick) != 0))
-		{
-			nicks[i] = user->nick;
-			i++;
-		}
-		list = list->next;
-	}
-
-	send_channel_modes (sess, tbuf, nicks, 0, i, '-', 'h');
+	multidata data;
 
+	data.nicks = nicks;
+	data.i = 0;
+	tree_foreach (sess->usertree, (tree_traverse_func *)mdehop_cb, &data);
+	send_channel_modes (sess, tbuf, nicks, 0, data.i, '-', 'h');
 	free (nicks);
 
 	return TRUE;
 }
 
+static int
+mdeop_cb (struct User *user, multidata *data)
+{
+	if (user->op && !user->me)
+	{
+		data->nicks[data->i] = user->nick;
+		data->i++;
+	}
+	return TRUE;
+}
+
 static int
 cmd_mdeop (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
-	struct User *user;
 	char **nicks = malloc (sizeof (char *) * sess->ops);
-	int i = 0;
-	GSList *list = sess->userlist;
-
-	while (list)
-	{
-		user = (struct User *) list->data;
-		if (user->op && (strcmp (user->nick, sess->server->nick) != 0))
-		{
-			nicks[i] = user->nick;
-			i++;
-		}
-		list = list->next;
-	}
-
-	send_channel_modes (sess, tbuf, nicks, 0, i, '-', 'o');
+	multidata data;
 
+	data.nicks = nicks;
+	data.i = 0;
+	tree_foreach (sess->usertree, (tree_traverse_func *)mdeop_cb, &data);
+	send_channel_modes (sess, tbuf, nicks, 0, data.i, '-', 'o');
 	free (nicks);
 
 	return TRUE;
 }
 
+static int
+mkick_cb (struct User *user, multidata *data)
+{
+	if (!user->op && !user->me)
+		data->sess->server->p_kick (data->sess->server, data->sess->channel, user->nick, data->reason);
+	return TRUE;
+}
+
+static int
+mkickops_cb (struct User *user, multidata *data)
+{
+	if (user->op && !user->me)
+		data->sess->server->p_kick (data->sess->server, data->sess->channel, user->nick, data->reason);
+	return TRUE;
+}
+
 static int
 cmd_mkick (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
-	struct User *user;
-	char *reason = word_eol[2];
-	GSList *list;
+	multidata data;
 
-	list = sess->userlist;
-	while (list)
-	{
-		user = (struct User *) list->data;
-		if (user->op)
-		{
-			if (strcmp (user->nick, sess->server->nick) != 0)
-			{
-				sess->server->p_kick (sess->server, sess->channel, user->nick, reason);
-			}
-		}
-		list = list->next;
-	}
-
-	list = sess->userlist;
-	while (list)
-	{
-		user = (struct User *) list->data;
-		if (!user->op)
-		{
-			if (strcmp (user->nick, sess->server->nick) != 0)
-			{
-				sess->server->p_kick (sess->server, sess->channel, user->nick, reason);
-			}
-		}
-		list = list->next;
-	}
+	data.sess = sess;
+	data.reason = word_eol[2];
+	tree_foreach (sess->usertree, (tree_traverse_func *)mkickops_cb, &data);
+	tree_foreach (sess->usertree, (tree_traverse_func *)mkick_cb, &data);
 
 	return TRUE;
 }
@@ -1883,6 +1884,33 @@ cmd_me (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return TRUE;
 }
 
+static int
+mop_cb (struct User *user, multidata *data)
+{
+	if (!user->op)
+	{
+		data->nicks[data->i] = user->nick;
+		data->i++;
+	}
+	return TRUE;
+}
+
+static int
+cmd_mop (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	char **nicks = malloc (sizeof (char *) * (sess->total - sess->ops));
+	multidata data;
+
+	data.nicks = nicks;
+	data.i = 0;
+	tree_foreach (sess->usertree, (tree_traverse_func *)mop_cb, &data);
+	send_channel_modes (sess, tbuf, nicks, 0, data.i, '+', 'o');
+
+	free (nicks);
+
+	return TRUE;
+}
+
 static int
 cmd_msg (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -2355,26 +2383,47 @@ cmd_unload (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return FALSE;
 }
 
+static int
+userlist_cb (struct User *user, session *sess)
+{
+	time_t lt;
+
+	if (!user->lasttalk)
+		lt = 0;
+	else
+		lt = time (0) - user->lasttalk;
+	PrintTextf (sess,
+				"\00306%s\t\00314[\00310%-38s\00314] \017ov\0033=\017%d%d lt\0033=\017%d\n",
+				user->nick, user->hostname, user->op, user->voice, lt);
+
+	return TRUE;
+}
+
 static int
 cmd_userlist (struct session *sess, char *tbuf, char *word[],
 				  char *word_eol[])
 {
-	struct User *user;
-	GSList *list;
-	int lt;
+	tree_foreach (sess->usertree, (tree_traverse_func *)userlist_cb, sess);
+	return TRUE;
+}
 
-	list = sess->userlist;
-	while (list)
+static int
+wallchop_cb (struct User *user, multidata *data)
+{
+	if (user->op)
 	{
-		user = list->data;
-		lt = time (0) - user->lasttalk;
-		if (!user->lasttalk)
-			lt = 0;
-		sprintf (tbuf,
-					"\00306%s\t\00314[\00310%-38s\00314] \017ov\0033=\017%d%d lt\0033=\017%d\n",
-					user->nick, user->hostname, user->op, user->voice, lt);
-		PrintText (sess, tbuf);
-		list = list->next;
+		if (data->i)
+			strcat (data->tbuf, ",");
+		strcat (data->tbuf, user->nick);
+		data->i++;
+	}
+	if (data->i == 5)
+	{
+		data->i = 0;
+		sprintf (data->tbuf + strlen (data->tbuf),
+					" :[@%s] %s", data->sess->channel, data->reason);
+		data->sess->server->p_raw (data->sess->server, data->tbuf);
+		strcpy (data->tbuf, "NOTICE ");
 	}
 
 	return TRUE;
@@ -2384,37 +2433,27 @@ static int
 cmd_wallchop (struct session *sess, char *tbuf, char *word[],
 				  char *word_eol[])
 {
-	int i = 0;
-	struct User *user;
-	GSList *list;
+	multidata data;
 
-	if (*word_eol[2])
+	if (!(*word_eol[2]))
+		return FALSE;
+
+	strcpy (tbuf, "NOTICE ");
+
+	data.reason = word_eol[2];
+	data.tbuf = tbuf;
+	data.i = 0;
+	data.sess = sess;
+	tree_foreach (sess->usertree, (tree_traverse_func*)wallchop_cb, &data);
+
+	if (data.i)
 	{
-		strcpy (tbuf, "NOTICE ");
-		list = sess->userlist;
-		while (list)
-		{
-			user = (struct User *) list->data;
-			if (user->op)
-			{
-				if (i)
-					strcat (tbuf, ",");
-				strcat (tbuf, user->nick);
-				i++;
-			}
-			if ((i == 5 || !list->next) && i)
-			{
-				i = 0;
-				sprintf (tbuf + strlen (tbuf),
-							" :[@%s] %s", sess->channel, word_eol[2]);
-				sess->server->p_raw (sess->server, tbuf);
-				strcpy (tbuf, "NOTICE ");
-			}
-			list = list->next;
-		}
-		return TRUE;
+		sprintf (tbuf + strlen (tbuf),
+					" :[@%s] %s", sess->channel, word_eol[2]);
+		sess->server->p_raw (sess->server, tbuf);
 	}
-	return FALSE;
+
+	return TRUE;
 }
 
 static int
@@ -2478,7 +2517,7 @@ cmd_voice (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	}
 }
 
-
+/* *MUST* be kept perfectly sorted for the bsearch to work */
 const struct commands xc_cmds[] = {
 	{"ADDBUTTON", cmd_addbutton, 0, 0,
 	 N_("ADDBUTTON <name> <action>, adds a button under the user-list")},
@@ -2489,16 +2528,14 @@ const struct commands xc_cmds[] = {
 	{"AWAY", cmd_away, 1, 0, N_("AWAY [<reason>], sets you away")},
 	{"BAN", cmd_ban, 1, 1,
 	 N_("BAN <mask> [<bantype>], bans everyone matching the mask from the current channel. If they are already on the channel this doesn't kick them (needs chanop)")},
-	{"UNBAN", cmd_unban, 1, 1,
-	 N_("UNBAN <mask> [<mask>...], unbans the specified masks.")},
 	{"CHARSET", cmd_charset, 0, 0, 0},
 	{"CLEAR", cmd_clear, 0, 0, N_("CLEAR, Clears the current text window")},
 	{"CLOSE", cmd_close, 0, 0, N_("CLOSE, Closes the current window/tab")},
 
-	{"CTCP", cmd_ctcp, 1, 0,
-	 N_("CTCP <nick> <message>, send the CTCP message to nick, common messages are VERSION and USERINFO")},
 	{"COUNTRY", cmd_country, 0, 0,
 	 N_("COUNTRY <code>, finds a country code, eg: au = australia")},
+	{"CTCP", cmd_ctcp, 1, 0,
+	 N_("CTCP <nick> <message>, send the CTCP message to nick, common messages are VERSION and USERINFO")},
 	{"CYCLE", cmd_cycle, 1, 1,
 	 N_("CYCLE, parts current channel and immediately rejoins")},
 	{"DCC", cmd_dcc, 0, 0,
@@ -2511,10 +2548,10 @@ const struct commands xc_cmds[] = {
 	 "         /dcc close send johnsmith file.tar.gz")},
 	{"DEBUG", cmd_debug, 0, 0, 0},
 
-	{"DELBUTTON", cmd_delbutton, 0, 0,
-	 N_("DELBUTTON <name>, deletes a button from under the user-list")},
 	{"DEHOP", cmd_dehop, 1, 1,
 	 N_("DEHOP <nick>, removes chanhalf-op status from the nick on the current channel (needs chanop)")},
+	{"DELBUTTON", cmd_delbutton, 0, 0,
+	 N_("DELBUTTON <name>, deletes a button from under the user-list")},
 	{"DEOP", cmd_deop, 1, 1,
 	 N_("DEOP <nick>, removes chanop status from the nick on the current channel (needs chanop)")},
 	{"DEVOICE", cmd_devoice, 1, 1,
@@ -2525,11 +2562,13 @@ const struct commands xc_cmds[] = {
 #ifndef WIN32
 	{"EXEC", cmd_exec, 0, 0,
 	 N_("EXEC [-o] <command>, runs the command. If -o flag is used then output is sent to current channel, else is printed to current text box")},
+#ifndef __EMX__
+	{"EXECCONT", cmd_execc, 0, 0, N_("EXECCONT, sends the process SIGCONT")},
+#endif
 	{"EXECKILL", cmd_execk, 0, 0,
 	 N_("EXECKILL [-9], kills a running exec in the current session. If -9 is given the process is SIGKILL'ed")},
 #ifndef __EMX__
 	{"EXECSTOP", cmd_execs, 0, 0, N_("EXECSTOP, sends the process SIGSTOP")},
-	{"EXECCONT", cmd_execc, 0, 0, N_("EXECCONT, sends the process SIGCONT")},
 	{"EXECWRITE", cmd_execw, 0, 0, N_("EXECWRITE, sends data to the processes stdin")},
 #endif
 #endif
@@ -2567,10 +2606,12 @@ const struct commands xc_cmds[] = {
 	 N_("MDEHOP, Mass deop's all chanhalf-ops in the current channel (needs chanop)")},
 	{"MDEOP", cmd_mdeop, 1, 1,
 	 N_("MDEOP, Mass deop's all chanops in the current channel (needs chanop)")},
-	{"MKICK", cmd_mkick, 1, 1,
-	 N_("MKICK, Mass kicks everyone except you in the current channel (needs chanop)")},
 	{"ME", cmd_me, 0, 0,
 	 N_("ME <action>, sends the action to the current channel (actions are written in the 3rd person, like /me jumps)")},
+	{"MKICK", cmd_mkick, 1, 1,
+	 N_("MKICK, Mass kicks everyone except you in the current channel (needs chanop)")},
+	{"MOP", cmd_mop, 1, 1,
+	 N_("MOP, Mass op's all users in the current channel (needs chanop)")},
 	{"MSG", cmd_msg, 0, 0, N_("MSG <nick> <message>, sends a private message")},
 
 	{"NAMES", cmd_names, 1, 0,
@@ -2607,8 +2648,6 @@ const struct commands xc_cmds[] = {
 
 	{"SAY", cmd_say, 0, 0,
 	 N_("SAY <text>, sends the text to the object in the current window")},
-	{"SET", cmd_set, 0, 0, N_("SET <variable> [<value>]")},
-	{"SETTAB", cmd_settab, 0, 0, ""},
 #ifdef USE_OPENSSL
 	{"SERVCHAN", cmd_servchan, 0, 0,
 	 N_("SERVCHAN [-ssl] <host> <port> <channel>, connects and joins a channel")},
@@ -2623,69 +2662,89 @@ const struct commands xc_cmds[] = {
 	{"SERVER", cmd_server, 0, 0,
 	 N_("SERVER <host> [<port>] [<password>], connects to a server, the default port is 6667")},
 #endif
+	{"SET", cmd_set, 0, 0, N_("SET <variable> [<value>]")},
+	{"SETTAB", cmd_settab, 0, 0, ""},
 	{"TOPIC", cmd_topic, 1, 1,
 	 N_("TOPIC [<topic>], sets the topic if one is given, else shows the current topic")},
+	{"UNBAN", cmd_unban, 1, 1,
+	 N_("UNBAN <mask> [<mask>...], unbans the specified masks.")},
 	{"UNIGNORE", cmd_unignore, 0, 0, N_("UNIGNORE <mask> [QUIET]")},
 	{"UNLOAD", cmd_unload, 0, 0, N_("UNLOAD <name>, unloads a plugin or script")},
 	{"USERLIST", cmd_userlist, 1, 1, 0},
-	{"WALLCHOP", cmd_wallchop, 1, 1,
-	 N_("WALLCHOP <message>, sends the message to all chanops on the current channel")},
-	{"WALLCHAN", cmd_wallchan, 1, 1,
-	 N_("WALLCHAN <message>, writes the message to all channels")},
 	{"VOICE", cmd_voice, 1, 1,
 	 N_("VOICE <nick>, gives voice status to someone (needs chanop)")},
+	{"WALLCHAN", cmd_wallchan, 1, 1,
+	 N_("WALLCHAN <message>, writes the message to all channels")},
+	{"WALLCHOP", cmd_wallchop, 1, 1,
+	 N_("WALLCHOP <message>, sends the message to all chanops on the current channel")},
 	{0, 0, 0, 0, 0}
 };
 
+
+static int
+command_compare (const void *a, const void *b)
+{
+	return strcasecmp (a, ((struct commands *)b)->name);
+}
+
+static struct commands *
+find_internal_command (char *name)
+{
+	/* the "-1" is to skip the NULL terminator */
+	return bsearch (name, xc_cmds, (sizeof (xc_cmds) /
+				sizeof (xc_cmds[0])) - 1, sizeof (xc_cmds[0]), command_compare);
+}
+
 static void
 help (session *sess, char *tbuf, char *helpcmd, int quiet)
 {
-	int i = 0;
+	struct commands *cmd;
 
 	if (plugin_show_help (sess, helpcmd))
 		return;
 
-	while (1)
+	cmd = find_internal_command (helpcmd);
+
+	if (cmd)
 	{
-		if (!xc_cmds[i].name)
-			break;
-		if (!strcasecmp (helpcmd, xc_cmds[i].name))
+		if (cmd->help)
 		{
-			if (xc_cmds[i].help)
-			{
-				sprintf (tbuf, _("Usage: %s\n"), _(xc_cmds[i].help));
-				PrintText (sess, tbuf);
-			} else
-			{
-				if (!quiet)
-					PrintText (sess, _("\nNo help available on that command.\n"));
-			}
-			return;
+			sprintf (tbuf, _("Usage: %s\n"), _(cmd->help));
+			PrintText (sess, tbuf);
+		} else
+		{
+			if (!quiet)
+				PrintText (sess, _("\nNo help available on that command.\n"));
 		}
-		i++;
+		return;
 	}
+
 	if (!quiet)
 		PrintText (sess, _("No such command.\n"));
 }
 
 /* inserts %a, %c, %d etc into buffer. Also handles &x %x for word/word_eol. *
+ *   returns 2 on buffer overflow
  *   returns 1 on success                                                    *
  *   returns 0 on bad-args-for-user-command                                  *
  * - word/word_eol args might be NULL                                        *
  * - this beast is used for UserCommands, UserlistButtons and CTCP replies   */
 
 int
-auto_insert (char *dest, unsigned char *src, char *word[], char *word_eol[],
-				 char *a, char *c, char *d, char *h, char *n, char *s)
+auto_insert (char *dest, int destlen, unsigned char *src, char *word[],
+				 char *word_eol[], char *a, char *c, char *d, char *h, char *n,
+				 char *s)
 {
 	int num;
-	char buf[4];
+	char buf[32];
 	time_t now;
 	struct tm *tm_ptr;
 	char *utf;
 	gsize utf_len;
 	char *orig = dest;
 
+	destlen--;
+
 	while (src[0])
 	{
 		if (src[0] == '%' || src[0] == '&')
@@ -2702,6 +2761,12 @@ auto_insert (char *dest, unsigned char *src, char *word[], char *word_eol[],
 					utf = g_locale_to_utf8 (dest, 1, 0, &utf_len, 0);
 					if (utf)
 					{
+						if ((dest - orig) + utf_len >= destlen)
+						{
+							g_free (utf);
+							return 2;
+						}
+
 						memcpy (dest, utf, utf_len);
 						g_free (utf);
 						dest += utf_len;
@@ -2722,11 +2787,8 @@ auto_insert (char *dest, unsigned char *src, char *word[], char *word_eol[],
 							utf = word_eol[num];
 
 						/* avoid recusive usercommand overflow */
-						if ((dest - orig) + strlen (utf) >= 2048)
-						{
-							PrintText (0, "UserCommand buffer overflow.\n");
-							return 0;
-						}
+						if ((dest - orig) + strlen (utf) >= destlen)
+							return 2;
 
 						strcpy (dest, utf);
 						dest += strlen (dest);
@@ -2737,57 +2799,66 @@ auto_insert (char *dest, unsigned char *src, char *word[], char *word_eol[],
 				if (src[0] == '&')
 					goto lamecode;
 				src++;
+				utf = NULL;
 				switch (src[0])
 				{
 				case '%':
+					if ((dest - orig) + 2 >= destlen)
+						return 2;
 					dest[0] = '%';
 					dest[1] = 0;
 					break;
 				case 'a':
-					strcpy (dest, a);
-					break;
+					utf = a; break;
 				case 'c':
-					strcpy (dest, c);
-					break;
+					utf = c; break;
 				case 'd':
-					strcpy (dest, d);
-					break;
+					utf = d; break;
 				case 'h':
-					strcpy (dest, h);
-					break;
+					utf = h; break;
 				case 'm':
-					strcpy (dest, get_cpu_str ());
-					break;
+					utf = get_cpu_str (); break;
 				case 'n':
-					strcpy (dest, n);
-					break;
+					utf = n; break;
 				case 's':
-					strcpy (dest, s);
-					break;
+					utf = s; break;
 				case 't':
 					now = time (0);
-					memcpy (dest, ctime (&now), 19);
-					dest[19] = 0;
+					utf = ctime (&now);
+					utf[19] = 0;
 					break;
 				case 'v':
-					strcpy (dest, VERSION);
+					utf = VERSION; break;
 					break;
 				case 'y':
 					now = time (0);
 					tm_ptr = localtime (&now);
-					sprintf (dest, "%4d%02d%02d", 1900 + tm_ptr->tm_year,
-								1 + tm_ptr->tm_mon, tm_ptr->tm_mday);
+					snprintf (buf, sizeof (buf), "%4d%02d%02d", 1900 +
+								 tm_ptr->tm_year, 1 + tm_ptr->tm_mon, tm_ptr->tm_mday);
+					utf = buf;
 					break;
 				default:
 					src--;
 					goto lamecode;
 				}
-				dest += strlen (dest);
+
+				if (utf)
+				{
+					if ((dest - orig) + strlen (utf) >= destlen)
+						return 2;
+					strcpy (dest, utf);
+					dest += strlen (dest);
+				}
+
 			}
 			src++;
 		} else
 		{
 			utf_len = g_utf8_skip[src[0]];
+
+			if ((dest - orig) + utf_len >= destlen)
+				return 2;
+
 			if (utf_len == 1)
 			{
 		 lamecode:
@@ -2892,6 +2963,39 @@ check_special_chars (char *cmd, int do_ascii) /* check for %X */
 	}
 }
 
+typedef struct
+{
+	char *nick;
+	int len;
+	struct User *best;
+	int bestlen;
+	char *space;
+	char *tbuf;
+} nickdata;
+
+static int
+nick_comp_cb (struct User *user, nickdata *data)
+{
+	int lenu;
+
+	if (!rfc_ncasecmp (user->nick, data->nick, data->len))
+	{
+		lenu = strlen (user->nick);
+		if (lenu == data->len)
+		{
+			sprintf (data->tbuf, "%s%s", user->nick, data->space);
+			data->len = -1;
+			return FALSE;
+		} else if (lenu < data->bestlen)
+		{
+			data->bestlen = lenu;
+			data->best = user;
+		}
+	}
+
+	return TRUE;
+}
+
 static void
 perform_nick_completion (struct session *sess, char *cmd, char *tbuf)
 {
@@ -2904,37 +3008,26 @@ perform_nick_completion (struct session *sess, char *cmd, char *tbuf)
 			len = (long) space - (long) cmd - 1;
 			if (len < NICKLEN)
 			{
-				struct User *user;
-				struct User *best = NULL;
-				int bestlen = INT_MAX, lenu;
 				char nick[NICKLEN];
-				GSList *list;
+				nickdata data;
 
 				memcpy (nick, cmd, len);
 				nick[len] = 0;
 
-				list = sess->userlist;
-				while (list)
+				data.nick = nick;
+				data.len = len;
+				data.bestlen = INT_MAX;
+				data.best = NULL;
+				data.tbuf = tbuf;
+				data.space = space - 1;
+				tree_foreach (sess->usertree, (tree_traverse_func *)nick_comp_cb, &data);
+
+				if (data.len == -1)
+					return;
+
+				if (data.best)
 				{
-					user = (struct User *) list->data;
-					if (!rfc_ncasecmp (user->nick, nick, len))
-					{
-						lenu = strlen (user->nick);
-						if (lenu == len)
-						{
-							sprintf (tbuf, "%s%s", user->nick, space - 1);
-							return;
-						} else if (lenu < bestlen)
-						{
-							bestlen = lenu;
-							best = user;
-						}
-					}
-					list = list->next;
-				}
-				if (best)
-				{
-					sprintf (tbuf, "%s%s", best->nick, space - 1);
+					sprintf (tbuf, "%s%s", data.best->nick, space - 1);
 					return;
 				}
 			}
@@ -2948,7 +3041,7 @@ static void
 user_command (session * sess, char *tbuf, char *cmd, char *word[],
 				  char *word_eol[])
 {
-	if (!auto_insert (tbuf, cmd, word, word_eol, "",
+	if (!auto_insert (tbuf, 2048, cmd, word, word_eol, "",
 							sess->channel, "", "", sess->server->nick, ""))
 	{
 		PrintText (sess, _("Bad arguments for user command.\n"));
@@ -3051,13 +3144,14 @@ int
 handle_command (session *sess, char *cmd, int check_spch)
 {
 	struct popup *pop;
-	int user_cmd = FALSE, i;
+	int user_cmd = FALSE;
 	GSList *list;
 	char pdibuf[2048];
 	char tbuf[4096];
 	char *word[PDIWORDS];
 	char *word_eol[PDIWORDS];
 	static int command_level = 0;
+	struct commands *int_cmd;
 
 	if (command_level > 99)
 	{
@@ -3093,44 +3187,36 @@ handle_command (session *sess, char *cmd, int check_spch)
 		goto xit;
 
 	/* now check internal commands */
-	i = 0;
-	while (1)
+	int_cmd = find_internal_command (word[1]);
+
+	if (int_cmd)
 	{
-		if (!xc_cmds[i].name)
-			break;
-		if (!strcasecmp (word[1], xc_cmds[i].name))
+		if (int_cmd->needserver && !sess->server->connected)
 		{
-			if (xc_cmds[i].needserver && !sess->server->connected)
-			{
-				notc_msg (sess);
-				goto xit;
-			}
-			if (xc_cmds[i].needchannel && !sess->channel[0])
-			{
-				notj_msg (sess);
-				goto xit;
-			}
-			switch (xc_cmds[i].callback (sess, tbuf, word, word_eol))
+			notc_msg (sess);
+		} else if (int_cmd->needchannel && !sess->channel[0])
+		{
+			notj_msg (sess);
+		} else
+		{
+			switch (int_cmd->callback (sess, tbuf, word, word_eol))
 			{
 			case FALSE:
-				help (sess, tbuf, xc_cmds[i].name, TRUE);
+				help (sess, tbuf, int_cmd->name, TRUE);
 				break;
 			case 2:
 				command_level--;
 				return FALSE;
 			}
-			goto xit;
 		}
-		i++;
-	}
-
-	/* unknown command, just send it to the server and hope */
-	if (!sess->server->connected)
+	} else
 	{
-		PrintText (sess, _("Unknown Command. Try /help\n"));
-		goto xit;
+		/* unknown command, just send it to the server and hope */
+		if (!sess->server->connected)
+			PrintText (sess, _("Unknown Command. Try /help\n"));
+		else
+			sess->server->p_raw (sess->server, cmd);
 	}
-	sess->server->p_raw (sess->server, cmd);
 
 xit:
 	command_level--;
@@ -3180,8 +3266,25 @@ handle_user_input (session *sess, char *text, int history, int nocommand)
 	return handle_command (sess, text + 1, TRUE);
 }
 
+/* changed by Steve Green. Macs sometimes paste with imbedded \r */
 void
 handle_multiline (session *sess, char *cmd, int history, int nocommand)
+{
+	while (*cmd)
+	{
+		char *cr = cmd + strcspn (cmd, "\n\r");
+		int end_of_string = *cr == 0;
+		*cr = 0;
+		if (!handle_user_input (sess, cmd, history, nocommand))
+			return;
+		if (end_of_string)
+			break;
+		cmd = cr + 1;
+	}
+}
+
+/*void
+handle_multiline (session *sess, char *cmd, int history, int nocommand)
 {
 	char *cr;
 
@@ -3205,4 +3308,4 @@ handle_multiline (session *sess, char *cmd, int history, int nocommand)
 	{
 		handle_user_input (sess, cmd, history, nocommand);
 	}
-}
+}*/
