@@ -66,6 +66,7 @@ struct _xchat_list
 	int type;			/* LIST_* */
 	GSList *pos;		/* current pos */
 	GSList *next;		/* next pos */
+	GSList *head;
 };
 
 typedef int (xchat_cmd_cb) (char *word[], char *word_eol[], void *user_data);
@@ -950,7 +951,7 @@ xchat_list_get (xchat_plugin *ph, const char *name)
 		if (is_session (ph->context))
 		{
 			list->type = LIST_USERS;
-			list->next = ph->context->userlist;
+			list->head = list->next = userlist_flat_list (ph->context);
 			break;
 		}	/* fall through */
 
@@ -965,6 +966,8 @@ xchat_list_get (xchat_plugin *ph, const char *name)
 void
 xchat_list_free (xchat_plugin *ph, xchat_list *xlist)
 {
+	if (xlist->type == LIST_USERS)
+		g_slist_free (xlist->head);
 	free (xlist);
 }
 
@@ -989,19 +992,19 @@ xchat_list_fields (xchat_plugin *ph, const char *name)
 	};
 	static const char *channels_fields[] =
 	{
-		"schannel",	"pcontext",	"sserver",	NULL
+		"schannel",	"pcontext", "snetwork", "sserver",	"stype",	NULL
 	};
 	static const char *ignore_fields[] =
 	{
-		"smask",		"iflags",	NULL
+		"iflags", "smask", NULL
 	};
 	static const char *users_fields[] =
 	{
-		"snick",		"shost",		"sprefix",	NULL
+		"shost", "snick", "sprefix", NULL
 	};
 	static const char *list_of_lists[] =
 	{
-		"channels",	"dcc",		"ignore",	"users",		NULL
+		"channels",	"dcc", "ignore", "users", NULL
 	};
 
 	switch (str_hash (name))
@@ -1038,6 +1041,8 @@ xchat_list_str (xchat_plugin *ph, xchat_list *xlist, const char *name)
 			return data;	/* this is a session * */
 		case 0xca022f43: /* server */
 			return ((session *)data)->server->servername;
+		case 0x6de15a2e:        /* network */
+			return ((session *)data)->server->networkname;
 		}
 		break;
 
@@ -1114,6 +1119,15 @@ xchat_list_int (xchat_plugin *ph, xchat_list *xlist, const char *name)
 			return ((struct ignore *)data)->type;
 		}
 		break;
+
+	case LIST_CHANNELS:
+		switch (hash)
+		{
+		case 0x368f3a:	/* type */
+			return ((struct session *)data)->type;
+		}
+		break;
+
 	}
 
 	return -1;
