@@ -314,6 +314,11 @@ process_numeric (session * sess, char *outbuf, int n,
 		inbound_005 (serv, word);
 		goto def;
 
+	case 263:	/*Server load is temporarily too heavy */
+		if (fe_is_chanwindow (sess->server))
+			fe_chan_list_end (sess->server);
+		goto def;
+
 	case 301:
 		inbound_away (serv, word[4],
 						(word_eol[5][0] == ':') ? word_eol[5] + 1 : word_eol[5]);
@@ -388,9 +393,8 @@ process_numeric (session * sess, char *outbuf, int n,
 			fe_add_chan_list (sess->server, word[4], word[5], word_eol[6] + 1);
 		} else
 		{
-			sprintf (outbuf, "%-16s %-7d %s\017\n",
-						word[4], atoi (word[5]), word_eol[6] + 1);
-			PrintText (sess, outbuf);
+			PrintTextf (sess, "%-16s %-7d %s\017\n",
+							word[4], atoi (word[5]), word_eol[6] + 1);
 		}
 		break;
 
@@ -417,7 +421,7 @@ process_numeric (session * sess, char *outbuf, int n,
 		fe_update_mode_buttons (sess, 'm', '-');
 		fe_update_mode_buttons (sess, 'l', '-');
 		fe_update_mode_buttons (sess, 'k', '-');
-		handle_mode (serv, outbuf, word, word_eol, "", TRUE);
+		handle_mode (serv, word, word_eol, "", TRUE);
 		break;
 
 	case 329:
@@ -450,7 +454,8 @@ process_numeric (session * sess, char *outbuf, int n,
 		{
 			if (!inbound_user_info (sess, outbuf, word[4], word[5], word[6],
 											word[7], word[8], word_eol[11]))
-				goto def;
+				EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, text, word[1],
+								 NULL, NULL, 0);
 		} else
 		{
 			if (!serv->p_cmp (word[8], serv->nick))
@@ -619,7 +624,7 @@ process_named_msg (session *sess, char *type, char *outbuf,
 	}
 	if (!strcmp ("MODE", type))
 	{
-		handle_mode (serv, outbuf, word, word_eol, nick, FALSE);	/* modes.c */
+		handle_mode (serv, word, word_eol, nick, FALSE);	/* modes.c */
 		return;
 	}
 	if (!strcmp ("NICK", type))
@@ -691,7 +696,7 @@ process_named_msg (session *sess, char *type, char *outbuf,
 	}
 	if (!strcmp ("PONG", type))
 	{
-		inbound_ping_reply (sess, outbuf,
+		inbound_ping_reply (serv->server_session, outbuf,
 							 (word[4][0] == ':') ? word[4] + 1 : word[4], word[3]);
 		return;
 	}
