@@ -154,31 +154,34 @@ text_word_check (char *word)
 	if ((word[0] == '@' || word[0] == '+') && word[1] == '#')
 		return WORD_CHANNEL;
 
-	if (word[0] == '#' && word[1] != '#' && word[1] != 0)
+	if ((word[0] == '#' || word[0] == '&') && word[1] != '#' && word[1] != 0)
 		return WORD_CHANNEL;
 
-	if (!strncasecmp (word, "irc://", 6))
+	if (!strncasecmp (word, "irc.", 4) && word[4] != '.')
 		return WORD_URL;
 
-	if (!strncasecmp (word, "irc.", 4))
+	if (!strncasecmp (word, "ftp.", 4) && word[4] != '.')
 		return WORD_URL;
 
-	if (!strncasecmp (word, "ftp.", 4))
+	if (!strncasecmp (word, "www.", 4) && word[4] != '.')
 		return WORD_URL;
 
-	if (!strncasecmp (word, "ftp:", 4))
+	if (!strncasecmp (word, "irc://", 6) && word[6] != 0)
 		return WORD_URL;
 
-	if (!strncasecmp (word, "www.", 4))
+	if (!strncasecmp (word, "ftp://", 6) && word[6] != 0)
 		return WORD_URL;
 
-	if (!strncasecmp (word, "http:", 5))
+	if (!strncasecmp (word, "http://", 7) && word[7] != 0)
 		return WORD_URL;
 
-	if (!strncasecmp (word, "gopher:", 7))
+	if (!strncasecmp (word, "file://", 7) && word[7] != 0)
 		return WORD_URL;
 
-	if (!strncasecmp (word, "https:", 6))
+	if (!strncasecmp (word, "https://", 8) && word[8] != 0)
+		return WORD_URL;
+
+	if (!strncasecmp (word, "gopher://", 9) && word[9] != 0)
 		return WORD_URL;
 
 	if (find_name (sess, word))
@@ -298,7 +301,7 @@ log_create_filename (char *buf, char *servname, char *channame, char *netname)
 #else
 		mkdir (buf, S_IRUSR | S_IWUSR | S_IXUSR);
 #endif
-	auto_insert (fn, prefs.logmask, NULL, NULL, "", channame, "", "", netname, servname);
+	auto_insert (fn, sizeof (fn), prefs.logmask, NULL, NULL, "", channame, "", "", netname, servname);
 	g_free (channame);
 
 	snprintf (buf, 512, "%s/xchatlogs/%s", get_xdir (), fn);
@@ -428,17 +431,26 @@ text_validate (char **text, int *len)
 {
 	char *utf;
 	gsize utf_len;
+	GError *error = NULL;
 
 	/* valid utf8? */
 	if (g_utf8_validate (*text, *len, 0))
 		return NULL;
 
 	/* maybe it's iso-8859-1 */
-	utf = g_convert (*text, *len, "UTF-8", "ISO-8859-1", 0, &utf_len, 0);
+	utf = g_convert (*text, *len, "UTF-8", "ISO-8859-1", 0, &utf_len, &error);
 	if (!utf)       /* should never happen; all text is iso-8859-1 valid */
 	{
-		*text = g_strdup ("%INVALID%");
-		*len = 8;
+		if (error)
+		{
+			*text = g_strdup_printf ("\0034ERROR\017\t%s\n", error->message);
+			*len = strlen (*text);
+			g_error_free (error);
+		} else
+		{
+			*text = g_strdup ("%INVALID%");
+			*len = 9;
+		}
 	} else
 	{
 		*text = utf;
