@@ -19,6 +19,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+
 #include "../../config.h"
 #ifdef HAVE_STRINGS_H
 #include <strings.h>
@@ -27,12 +28,13 @@
 #include "modes.h"
 #include "fe.h"
 #include "notify.h"
+#include "tree.h"
 #include "xchatc.h"
 #include "util.h"
 
 
 static int
-nick_cmp_az_ops (struct User *user1, struct User *user2)
+nick_cmp_az_ops (server *serv, struct User *user1, struct User *user2)
 {
 	unsigned int access1 = user1->access;
 	unsigned int access2 = user2->access;
@@ -51,59 +53,49 @@ nick_cmp_az_ops (struct User *user1, struct User *user2)
 		}
 	}
 
-	return rfc_casecmp (user1->nick, user2->nick);
+	return serv->p_cmp (user1->nick, user2->nick);
 }
 
 static int
-nick_cmp (struct User *user1, struct User *user2)
+nick_cmp_alpha (struct User *user1, struct User *user2, server *serv)
+{
+	return serv->p_cmp (user1->nick, user2->nick);
+}
+
+static int
+nick_cmp (struct User *user1, struct User *user2, server *serv)
 {
 	switch (prefs.userlist_sort)
 	{
 	case 0:
-		return nick_cmp_az_ops (user1, user2);
+		return nick_cmp_az_ops (serv, user1, user2);
 	case 1:
-		return rfc_casecmp (user1->nick, user2->nick);
+		return serv->p_cmp (user1->nick, user2->nick);
 	case 2:
-		return -1 * nick_cmp_az_ops (user1, user2);
+		return -1 * nick_cmp_az_ops (serv, user1, user2);
 	case 3:
-		return -1 * rfc_casecmp (user1->nick, user2->nick);
+		return -1 * serv->p_cmp (user1->nick, user2->nick);
 	default:
 		return 1;
 	}
 }
 
+/*
+ insert name in appropriate place in linked list. Returns row number or:
+  -1: duplicate
+*/
+
 static int
-userlist_insertname (struct session *sess, struct User *newuser)
+userlist_insertname (session *sess, struct User *newuser)
 {
-	int row = 0;
-	struct User *user;
-	GSList *list = sess->userlist;
-
-	while (list)
+	if (!sess->usertree)
 	{
-		user = (struct User *) list->data;
-		if (nick_cmp (newuser, user) < 1)
-		{
-			sess->userlist = g_slist_insert (sess->userlist, newuser, row);
-			return row;
-		}
-		row++;
-		list = list->next;
+		sess->usertree = tree_new ((tree_cmp_func *)nick_cmp, sess->server);
+		sess->usertree_alpha = tree_new ((tree_cmp_func *)nick_cmp_alpha, sess->server);
 	}
-	sess->userlist = g_slist_append (sess->userlist, newuser);
-	return -1;
-}
 
-static void
-update_entry (struct session *sess, struct User *user)
-{
-	int row;
-
-	sess->userlist = g_slist_remove (sess->userlist, user);
-	row = userlist_insertname (sess, user);
-
-	fe_userlist_move (sess, user, row);
-	fe_userlist_numbers (sess);
+	tree_insert (sess->usertree_alpha, newuser);
+	return tree_insert (sess->usertree, newuser);
 }
 
 int
@@ -111,6 +103,7 @@ userlist_add_hostname (struct session *sess, char *nick, char *hostname,
 							  char *realname, char *servername)
 {
 	struct User *user;
+	int pos;
 
 	user = find_name (sess, nick);
 	if (user && !user->hostname)
@@ -121,14 +114,17 @@ userlist_add_hostname (struct session *sess, char *nick, char *hostname,
 		if (!user->servername)
 			user->servername = strdup (servername);
 		if (prefs.showhostname_in_userlist)
-			update_entry (sess, user);
+		{
+			tree_remove (sess->usertree, user, &pos);
+			fe_userlist_move (sess, user, tree_insert (sess->usertree, user));
+		}
 		return 1;
 	}
 	return 0;
 }
 
-static void
-free_user (struct User *user, GSList **list)
+static int
+free_user (struct User *user, gpointer data)
 {
 	if (user->realname)
 		free (user->realname);
@@ -137,19 +133,20 @@ free_user (struct User *user, GSList **list)
 	if (user->servername)
 		free (user->servername);
 	free (user);
-	*list = g_slist_remove (*list, user);
+
+	return TRUE;
 }
 
 void
 free_userlist (session *sess)
 {
-	struct User *user;
+	tree_foreach (sess->usertree, (tree_traverse_func *)free_user, NULL);
+	tree_destroy (sess->usertree);
+	tree_destroy (sess->usertree_alpha);
+
+	sess->usertree = NULL;
+	sess->usertree_alpha = NULL;
 
-	while (sess->userlist)
-	{
-		user = (struct User *) sess->userlist->data;
-		free_user (user, &sess->userlist);
-	}
 	sess->ops = 0;
 	sess->hops = 0;
 	sess->voices = 0;
@@ -164,21 +161,22 @@ clear_user_list (session *sess)
 	fe_userlist_numbers (sess);
 }
 
+static int
+find_cmp (const char *name, struct User *user, server *serv)
+{
+	return serv->p_cmp ((char *)name, user->nick);
+}
+
 struct User *
 find_name (struct session *sess, char *name)
 {
-	struct User *user;
-	GSList *list;
+	int pos;
 
-	list = sess->userlist;
-	while (list)
-	{
-		user = (struct User *) list->data;
-		if (!rfc_casecmp (user->nick, name))
-			return user;
-		list = list->next;
-	}
-	return FALSE;
+	if (sess->usertree_alpha)
+		return tree_find (sess->usertree_alpha, name,
+								(tree_cmp_func *)find_cmp, sess->server, &pos);
+
+	return NULL;
 }
 
 struct User *
@@ -205,12 +203,8 @@ static void
 update_counts (session *sess, struct User *user, char prefix,
 					int level, int offset)
 {
-	/* the only ones the userlist supports with icons */
 	switch (prefix)
 	{
-	case '*':
-		user->admin = level;
-		break;
 	case '@':
 		user->op = level;
 		sess->ops += offset;
@@ -232,6 +226,7 @@ ul_update_entry (session *sess, char *name, char mode, char sign)
 	int access;
 	int offset = 0;
 	int level;
+	int pos;
 	char prefix;
 	struct User *user;
 
@@ -239,6 +234,10 @@ ul_update_entry (session *sess, char *name, char mode, char sign)
 	if (!user)
 		return;
 
+	/* remove from binary trees, before we loose track of it */
+	tree_remove (sess->usertree, user, &pos);
+	tree_remove (sess->usertree_alpha, user, &pos);
+
 	/* which bit number is affected? */
 	access = mode_access (sess->server, mode, &prefix);
 
@@ -266,24 +265,44 @@ ul_update_entry (session *sess, char *name, char mode, char sign)
 	/* update the various counts using the CHANGED prefix only */
 	update_counts (sess, user, prefix, level, offset);
 
-	update_entry (sess, user);
+	/* insert it back into its new place */
+	tree_insert (sess->usertree_alpha, user);
+	pos = tree_insert (sess->usertree, user);
+
+	/* let GTK move it too */
+	fe_userlist_move (sess, user, pos);
+	fe_userlist_numbers (sess);
 }
 
-void
+int
 change_nick (struct session *sess, char *oldname, char *newname)
 {
 	struct User *user = find_name (sess, oldname);
+	int pos;
+
 	if (user)
 	{
+		tree_remove (sess->usertree, user, &pos);
+		tree_remove (sess->usertree_alpha, user, &pos);
+
 		safe_strcpy (user->nick, newname, NICKLEN);
-		update_entry (sess, user);
+
+		tree_insert (sess->usertree_alpha, user);
+
+		fe_userlist_move (sess, user, tree_insert (sess->usertree, user));
+		fe_userlist_numbers (sess);
+
+		return 1;
 	}
+
+	return 0;
 }
 
 int
 sub_name (struct session *sess, char *name)
 {
 	struct User *user;
+	int pos;
 
 	user = find_name (sess, name);
 	if (!user)
@@ -299,7 +318,9 @@ sub_name (struct session *sess, char *name)
 	fe_userlist_numbers (sess);
 	fe_userlist_remove (sess, user);
 
-	free_user (user, &sess->userlist);
+	tree_remove (sess->usertree, user, &pos);
+	tree_remove (sess->usertree_alpha, user, &pos);
+	free_user (user, NULL);
 
 	return TRUE;
 }
@@ -315,10 +336,6 @@ add_name (struct session *sess, char *name, char *hostname)
 
 	notify_set_online (sess->server, name + prefix_chars);
 
-	/* dup? */
-	if (find_name (sess, name + prefix_chars))
-		return;
-
 	user = malloc (sizeof (struct User));
 	memset (user, 0, sizeof (struct User));
 
@@ -328,6 +345,24 @@ add_name (struct session *sess, char *name, char *hostname)
 	if (prefix_chars)
 		user->prefix[0] = name[0];
 
+	/* add it to our linked list */
+	if (hostname)
+		user->hostname = strdup (hostname);
+	safe_strcpy (user->nick, name + prefix_chars, NICKLEN);
+	/* is it me? */
+	if (!sess->server->p_cmp (user->nick, sess->server->nick))
+		user->me = TRUE;
+	row = userlist_insertname (sess, user);
+
+	/* duplicate? some broken servers trigger this */
+	if (row == -1)
+	{
+		if (user->hostname)
+			free (user->hostname);
+		free (user);
+		return;
+	}
+
 	sess->total++;
 
 	/* most ircds don't support multiple modechars infront of the nickname
@@ -339,11 +374,6 @@ add_name (struct session *sess, char *name, char *hostname)
 		prefix_chars--;
 	}
 
-	if (hostname)
-		user->hostname = strdup (hostname);
-	safe_strcpy (user->nick, name, NICKLEN);
-	row = userlist_insertname (sess, user);
-
 	fe_userlist_insert (sess, user, row, FALSE);
 	fe_userlist_numbers (sess);
 }
@@ -351,6 +381,7 @@ add_name (struct session *sess, char *name, char *hostname)
 void
 update_all_of (char *name)
 {
+#if 0
 	struct User *user;
 	struct session *sess;
 	GSList *list = sess_list;
@@ -364,4 +395,21 @@ update_all_of (char *name)
 		}
 		list = list->next;
 	}
+#endif
+}
+
+static gboolean
+flat_cb (struct User *user, GSList **list)
+{
+	*list = g_slist_prepend (*list, user);
+	return TRUE;
+}
+
+GSList *
+userlist_flat_list (session *sess)
+{
+	GSList *list = NULL;
+
+	tree_foreach (sess->usertree, (tree_traverse_func *)flat_cb, &list);
+	return g_slist_reverse (list);
 }
