@@ -13,9 +13,10 @@ struct User
 	unsigned int op:1;
 	unsigned int hop:1;
 	unsigned int voice:1;
+	unsigned int me:1;
 };
 
-#define USERACCESS_SIZE 16
+#define USERACCESS_SIZE 12
 
 int userlist_add_hostname (struct session *sess, char *nick,
 											 char *hostname, char *realname,
@@ -27,6 +28,7 @@ void clear_user_list (struct session *sess);
 void free_userlist (struct session *sess);
 void add_name (struct session *sess, char *name, char *hostname);
 int sub_name (struct session *sess, char *name);
-void change_nick (struct session *sess, char *oldname, char *newname);
+int change_nick (struct session *sess, char *oldname, char *newname);
 void ul_update_entry (session *sess, char *name, char mode, char sign);
 void update_all_of (char *name);
+GSList *userlist_flat_list (session *sess);
