@@ -730,7 +730,16 @@ xchat_init (void)
 #ifdef WIN32
 	WSADATA wsadata;
 
+#ifdef USE_IPV6
+	if (WSAStartup(0x0202, &wsadata) != 0)
+	{
+		MessageBox (NULL, "Cannot find winsock 2.2+", "Error", MB_OK);
+		exit (0);
+	}
+#else
 	WSAStartup(0x0101, &wsadata);
+#endif
+
 #else
 	struct sigaction act;
 
@@ -831,10 +840,10 @@ xchat_init (void)
 		_("Kick/Ban"),
 		_("Kick"),
 		_("Ban"),
-		_("Kick"),
-		_("Kick"),
-		_("Kick"),
-		_("Kick"),
+		_("Ban"),
+		_("Ban"),
+		_("Ban"),
+		_("Ban"),
 		_("KickBan"),
 		_("KickBan"),
 		_("KickBan"),
