@@ -167,6 +167,7 @@ struct xchatprefs
 	unsigned long dcc_ip;
 	char dcc_ip_str[16];
 
+	unsigned int tab_dnd;
 	unsigned int mainwindow_save;
 	unsigned int perc_color;
 	unsigned int perc_ascii;
@@ -265,7 +266,8 @@ struct xchatprefs
 typedef struct session
 {
 	struct server *server;
-	GSList *userlist;
+	void *usertree_alpha;			/* pure alphabetical tree */
+	void *usertree;					/* ordered with Ops first */
 	char channel[CHANLEN];
 	char waitchannel[CHANLEN];		  /* waiting to join this channel */
 	char willjoinchannel[CHANLEN];	  /* /join done for this channel */
