@@ -68,6 +68,7 @@ static int ignore_chanmode = FALSE;
 static const char chan_flags[] = { 't', 'n', 's', 'i', 'p', 'm', 'l', 'k' };
 
 static GtkWidget *active_tab = NULL;	/* active tab - toggle button */
+GtkWidget *parent_window = NULL;			/* the master window */
 
 GtkStyle *input_style;
 
@@ -281,6 +282,10 @@ mg_show_generic_tab (GtkWidget *button)
 {
 	GtkWidget *box;
 	int num;
+	GtkWidget *f = NULL;
+
+	if (current_sess && GTK_WIDGET_HAS_FOCUS (current_sess->gui->input_box))
+		f = current_sess->gui->input_box;
 
 	box = g_object_get_data (G_OBJECT (button), "box");
 	num = gtk_notebook_page_num (GTK_NOTEBOOK (mg_gui->note_book), box);
@@ -288,6 +293,9 @@ mg_show_generic_tab (GtkWidget *button)
 	gtk_window_set_title (GTK_WINDOW (mg_gui->window),
 								 g_object_get_data (G_OBJECT (button), "title"));
 	gtk_widget_set_sensitive (mg_gui->menu, FALSE);
+
+	if (f)
+		gtk_widget_grab_focus (f);
 }
 
 /* a channel has been focused */
@@ -536,7 +544,7 @@ mg_populate (session *sess)
 	if (gui->pane)
 	{
 		/* no change? then there will be no expose, so render now */
-		if (vis != gui->ul_hidden)
+		if (vis != gui->ul_hidden && gui->user_box->allocation.width > 1)
 			render = FALSE;
 	} else
 	{
@@ -707,6 +715,7 @@ mg_ircdestroy (GtkWidget *tab, session *sess)
 	gtk_widget_destroy (mg_gui->window);
 	active_tab = NULL;
 	mg_gui = NULL;
+	parent_window = NULL;
 }
 
 /* a tab has been destroyed */
@@ -989,7 +998,7 @@ mg_add_chan (session *sess)
 
 	sess->res->tab = tab_group_add (sess->gui->tabs_box, name, sess->server,
 											  sess, mg_tab_press_cb, mg_link_cb,
-											  prefs.truncchans);
+											  prefs.truncchans, prefs.tab_dnd);
 	g_object_set_data (G_OBJECT (sess->res->tab), "sess", sess);
 
 	if (newmsg_list == NULL)
@@ -1169,7 +1178,7 @@ mg_x_click_cb (GtkWidget *button, gpointer userdata)
 	int page;
 
 	/* this, to handle CTRL-W while in non-irc tab */
-	if (current_sess->gui->is_tab && mg_gui)
+	if (current_sess && current_sess->gui->is_tab && mg_gui)
 	{
 		page = gtk_notebook_get_current_page (GTK_NOTEBOOK (mg_gui->note_book));
 		if (page != 0)
@@ -1489,7 +1498,7 @@ mg_dialog_button_cb (GtkWidget *wid, char *cmd)
 	if (topic)
 		host = topic + 1;
 
-	auto_insert (buf, cmd, 0, 0, "", "", "", host, "", current_sess->channel);
+	auto_insert (buf, sizeof (buf), cmd, 0, 0, "", "", "", host, "", current_sess->channel);
 
 	handle_command (current_sess, buf, TRUE);
 
@@ -2022,7 +2031,7 @@ mg_create_menu (session_gui *gui, GtkWidget *box, int away_state)
 										 accel_group);
 	g_object_unref (accel_group);
 
-	gui->menu = menu_create_main (accel_group, TRUE, away_state,
+	gui->menu = menu_create_main (accel_group, TRUE, away_state, !gui->is_tab,
 											&gui->away_item, &gui->user_menu);
 	gtk_box_pack_start (GTK_BOX (box), gui->menu, 0, 0, 0);
 	gtk_box_reorder_child (GTK_BOX (box), gui->menu, 0);
@@ -2035,8 +2044,8 @@ mg_create_topwindow (session *sess)
 	GtkWidget *vbox;
 	GtkWidget *vvbox;
 
-	win = gtkutil_window_new ("X-Chat ["VERSION"]", prefs.mainwindow_width,
-									  prefs.mainwindow_height, FALSE);
+	win = gtkutil_window_new ("X-Chat ["VERSION"]", NULL, prefs.mainwindow_width,
+									  prefs.mainwindow_height, 0);
 	sess->gui->window = win;
 	gtk_container_set_border_width (GTK_CONTAINER (win), 2);
 	g_signal_connect (G_OBJECT (win), "focus_in_event",
@@ -2061,6 +2070,7 @@ mg_create_topwindow (session *sess)
 	{
 		sess->res->buffer = gtk_xtext_buffer_new (GTK_XTEXT (sess->gui->xtext));
 		gtk_xtext_buffer_show (GTK_XTEXT (sess->gui->xtext), sess->res->buffer, TRUE);
+		gtk_xtext_set_time_stamp (sess->res->buffer, prefs.timestamp);
 		sess->res->user_model = userlist_create_model ();
 	}
 
@@ -2134,6 +2144,7 @@ mg_tabwindow_kill_cb (GtkWidget *win, gpointer userdata)
 	current_tab = NULL;
 	active_tab = NULL;
 	mg_gui = NULL;
+	parent_window = NULL;
 }
 
 static gboolean
@@ -2167,8 +2178,8 @@ mg_create_tabwindow (session *sess)
 	GtkWidget *table;
 	GtkWidget *book;
 
-	win = gtkutil_window_new ("X-Chat ["VERSION"]", prefs.mainwindow_width,
-									  prefs.mainwindow_height, FALSE);
+	win = gtkutil_window_new ("X-Chat ["VERSION"]", NULL, prefs.mainwindow_width,
+									  prefs.mainwindow_height, 0);
 	sess->gui->window = win;
 	gtk_window_move (GTK_WINDOW (win), prefs.mainwindow_left,
 						  prefs.mainwindow_top);
@@ -2259,7 +2270,8 @@ mg_add_generic_tab (char *name, char *title, void *family, GtkWidget *box)
 	gtk_widget_show (box);
 
 	but = tab_group_add (mg_gui->tabs_box, name, family, NULL,
-								mg_tab_press_cb, mg_link_cb, prefs.truncchans);
+								mg_tab_press_cb, mg_link_cb, prefs.truncchans,
+								prefs.tab_dnd);
 	g_object_set_data (G_OBJECT (but), "title", strdup (title));
 	g_object_set_data (G_OBJECT (but), "box", box);
 	g_object_set_data (G_OBJECT (but), "sess", NULL);
@@ -2354,17 +2366,6 @@ fe_dlgbuttons_update (session *sess)
 		gtk_widget_hide (current_tab->gui->dialogbutton_box);
 }
 
-void
-fe_change_nick (server *serv, char *nick, char *newnick)
-{
-	session *sess = find_dialog (serv, nick);
-	if (sess)
-	{
-		safe_strcpy (sess->channel, newnick, NICKLEN);
-		fe_set_title (sess);
-	}
-}
-
 void
 fe_update_mode_buttons (session *sess, char mode, char sign)
 {
@@ -2474,6 +2475,7 @@ mg_changui_new (session *sess, restore_gui *res, int tab)
 		sess->gui = gui;
 		mg_create_tabwindow (sess);
 		mg_gui = gui;
+		parent_window = gui->window;
 	} else
 	{
 		sess->gui = gui = mg_gui;
@@ -2502,7 +2504,7 @@ mg_create_generic_tab (char *name, char *title, int force_toplevel,
 
 	if (force_toplevel || !prefs.windows_as_tabs)
 	{
-		win = gtkutil_window_new (title, width, height, TRUE);
+		win = gtkutil_window_new (title, name, width, height, 3);
 		vbox = gtk_vbox_new (0, 0);
 		*vbox_ret = vbox;
 		gtk_container_add (GTK_CONTAINER (win), vbox);
