@@ -139,6 +139,7 @@ nick_command_parse (session *sess, char *cmd, char *nick, char *allnick)
 	char *buf;
 	char *host = _("Host unknown");
 	struct User *user;
+	int len;
 
 /*	if (sess->type == SESS_DIALOG)
 	{
@@ -154,9 +155,10 @@ nick_command_parse (session *sess, char *cmd, char *nick, char *allnick)
 	}
 
 	/* this can't overflow, since popup->cmd is only 256 */
-	buf = malloc (strlen (cmd) + strlen (nick) + strlen (allnick) + 512);
+	len = strlen (cmd) + strlen (nick) + strlen (allnick) + 512;
+	buf = malloc (len);
 
-	auto_insert (buf, cmd, 0, 0, allnick, sess->channel, "", host,
+	auto_insert (buf, len, cmd, 0, 0, allnick, sess->channel, "", host,
 						sess->server->nick, nick);
 
 	nick_command (sess, buf);
@@ -533,7 +535,7 @@ menu_middlemenu (session *sess, GdkEventButton *event)
 	GtkAccelGroup *accel_group;
 
 	accel_group = gtk_accel_group_new ();
-	menu = menu_create_main (accel_group, FALSE, sess->server->is_away, &away, &user);
+	menu = menu_create_main (accel_group, FALSE, sess->server->is_away, !sess->gui->is_tab, &away, &user);
 
 	menu_quick_item (0, 0, menu, 1, 0);	/* sep */
 
@@ -1191,7 +1193,7 @@ static struct mymenu mymenu[] = {
 #else
 #define menuoffset 2
 #endif
-	{N_("Detach Tab"), menu_detach, GTK_STOCK_REDO, M_MENUSTOCK, 0, 1, GDK_I},
+	{0, menu_detach, GTK_STOCK_REDO, M_MENUSTOCK, 0, 1, GDK_I},	/* 14 */
 	{N_("Close Tab"), menu_close, GTK_STOCK_CLOSE, M_MENUSTOCK, 0, 1, GDK_w},
 	{0, 0, 0, M_SEP, 0, 0},
 	{N_("Quit"), mg_safe_quit, GTK_STOCK_QUIT, M_MENUSTOCK, 0, 1, GDK_q},	/* 17 */
@@ -1277,7 +1279,7 @@ create_icon_menu (char *labeltext, void *stock_name, int is_stock)
 }
 
 GtkWidget *
-menu_create_main (void *accel_group, int bar, int away,
+menu_create_main (void *accel_group, int bar, int away, int toplevel,
 						GtkWidget **away_item_ret, GtkWidget **user_menu_ret)
 {
 	int i = 0;
@@ -1305,6 +1307,11 @@ menu_create_main (void *accel_group, int bar, int away,
 	mymenu[31-menuoffset].state = prefs.autodccsend;
 	/*mymenu[60-menuoffset].state = prefs.autosave;*/
 
+	if (!toplevel)
+		mymenu[14-menuoffset].text = _("Detach Tab");
+	else
+		mymenu[14-menuoffset].text = _("Attach Tab");
+
 	while (1)
 	{
 		switch (mymenu[i].type)
@@ -1334,7 +1341,9 @@ normalitem:
 			if (mymenu[i].key != 0)
 				gtk_widget_add_accelerator (item, "activate", accel_group,
 										mymenu[i].key,
-										mymenu[i].key == GDK_F1 ? 0 : GDK_CONTROL_MASK,
+										mymenu[i].key == GDK_F1 ? 0 :
+										mymenu[i].key == GDK_w ? GDK_CONTROL_MASK 
+										| GDK_SHIFT_MASK : GDK_CONTROL_MASK,
 										GTK_ACCEL_VISIBLE);
 			if (mymenu[i].callback)
 				g_signal_connect (G_OBJECT (item), "activate",
