@@ -16,6 +16,8 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#define GTK_DISABLE_DEPRECATED
+
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -26,6 +28,12 @@
 #include <gtk/gtkclist.h>
 #include <gtk/gtkstock.h>
 #include <gtk/gtkbox.h>
+#include <gtk/gtkscrolledwindow.h>
+
+#include <gtk/gtkliststore.h>
+#include <gtk/gtktreeview.h>
+#include <gtk/gtktreeselection.h>
+#include <gtk/gtkcellrenderertext.h>
 
 #include "../common/xchat.h"
 #define PLUGIN_C
@@ -37,11 +45,43 @@ typedef struct session xchat_context;
 #include "../common/xchatc.h"
 #include "gtkutil.h"
 
+/* model for the plugin treeview */
+enum
+{
+	NAME_COLUMN,
+	VERSION_COLUMN,
+	FILE_COLUMN,
+	DESC_COLUMN,
+	N_COLUMNS
+};
 
 static GtkWidget *plugin_window = NULL;
-static GtkWidget *plugin_clist;
 
 
+static GtkWidget *
+plugingui_treeview_new (GtkWidget *box)
+{
+	GtkListStore *store;
+	GtkWidget *view;
+	GtkTreeViewColumn *col;
+	int col_id;
+
+	store = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING,
+	                            G_TYPE_STRING, G_TYPE_STRING);
+	g_return_val_if_fail (store != NULL, NULL);
+	view = gtkutil_treeview_new (box, GTK_TREE_MODEL (store), NULL,
+	                             NAME_COLUMN, _("Name"),
+	                             VERSION_COLUMN, _("Version"),
+	                             FILE_COLUMN, _("File"),
+	                             DESC_COLUMN, _("Description"), -1);
+	for (col_id=0; (col = gtk_tree_view_get_column (GTK_TREE_VIEW (view), col_id));
+	     col_id++)
+			gtk_tree_view_column_set_alignment (col, 0.5);
+
+	gtk_widget_show (view);
+	return view;
+}
+
 static void
 plugingui_close_button (GtkWidget * wid, gpointer a)
 {
@@ -60,25 +100,29 @@ void
 fe_pluginlist_update (void)
 {
 	xchat_plugin *pl;
-	gchar *entry[4];
 	GSList *list;
+	GtkTreeView *view;
+	GtkListStore *store;
+	GtkTreeIter iter;
 
 	if (!plugin_window)
 		return;
 
-	gtk_clist_clear (GTK_CLIST (plugin_clist));
+	view = g_object_get_data (G_OBJECT (plugin_window), "view");
+	store = GTK_LIST_STORE (gtk_tree_view_get_model (view));
+	gtk_list_store_clear (store);
 
 	list = plugin_list;
 	while (list)
 	{
 		pl = list->data;
-		entry[1] = pl->version;
-		if (entry[1][0] != 0)
+		if (pl->version[0] != 0)
 		{
-			entry[0] = pl->name;
-			entry[2] = file_part (pl->filename);
-			entry[3] = pl->desc;
-			gtk_clist_append (GTK_CLIST (plugin_clist), entry);
+			gtk_list_store_append (store, &iter);
+			gtk_list_store_set (store, &iter, NAME_COLUMN, pl->name,
+			                    VERSION_COLUMN, pl->version,
+			                    FILE_COLUMN, file_part (pl->filename),
+			                    DESC_COLUMN, pl->desc, -1);
 		}
 		list = list->next;
 	}
@@ -116,14 +160,16 @@ plugingui_loadbutton_cb (GtkWidget * wid, gpointer unused)
 static void
 plugingui_unload (GtkWidget * wid, gpointer unused)
 {
-	int row, len;
+	int len;
 	char *modname, *file, *buf;
-
-	row = gtkutil_clist_selection (plugin_clist);
-	if (row == -1)
+	GtkTreeView *view;
+	GtkTreeIter iter;
+	
+	view = g_object_get_data (G_OBJECT (plugin_window), "view");
+	if (!gtkutil_treeview_get_selected (view, &iter, NAME_COLUMN, &modname,
+	                                    FILE_COLUMN, &file, -1))
 		return;
-	gtk_clist_get_text (GTK_CLIST (plugin_clist), row, 0, &modname);
-	gtk_clist_get_text (GTK_CLIST (plugin_clist), row, 2, &file);
+
 	len = strlen (file);
 #ifdef WIN32
 	if (len > 4 && strcasecmp (file + len - 4, ".dll") == 0)
@@ -144,13 +190,16 @@ plugingui_unload (GtkWidget * wid, gpointer unused)
 		handle_command (current_sess, buf, FALSE);
 		free (buf);
 	}
+
+	g_free (modname);
+	g_free (file);
 }
 
 void
 plugingui_open (void)
 {
-	gchar *titles[] = { _("Name"), _("Version"), _("File"), _("Description") };
-	GtkWidget *okb;
+	GtkWidget *view;
+	GtkWidget *vbox, *action_area;
 
 	if (plugin_window)
 	{
@@ -161,37 +210,26 @@ plugingui_open (void)
 	plugin_window = gtk_dialog_new ();
 	g_signal_connect (G_OBJECT (plugin_window), "destroy",
 							G_CALLBACK (plugingui_close), 0);
-	gtk_widget_set_usize (plugin_window, 360, 200);
-	gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (plugin_window)->vbox),
-											  4);
+	gtk_window_set_default_size (GTK_WINDOW (plugin_window), 450, 200);
+	vbox = GTK_DIALOG (plugin_window)->vbox;
+	action_area = GTK_DIALOG (plugin_window)->action_area;
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
 	gtk_window_set_position (GTK_WINDOW (plugin_window), GTK_WIN_POS_CENTER);
 	gtk_window_set_title (GTK_WINDOW (plugin_window), _("X-Chat: Plugins and Scripts"));
 
-	plugin_clist = gtkutil_clist_new (4, titles,
-						 						 GTK_DIALOG (plugin_window)->vbox,
-						 						 GTK_POLICY_AUTOMATIC, 0, 0, 0, 0,
-												 GTK_SELECTION_BROWSE);
-	gtk_clist_column_titles_passive (GTK_CLIST (plugin_clist));
-	gtk_widget_show (plugin_clist);
+	view = plugingui_treeview_new (vbox);
+	g_object_set_data (G_OBJECT (plugin_window), "view", view);
 
-	gtk_clist_set_column_auto_resize (GTK_CLIST (plugin_clist), 0, TRUE);
-	gtk_clist_set_column_auto_resize (GTK_CLIST (plugin_clist), 2, TRUE);
+	gtkutil_button (action_area, GTK_STOCK_REVERT_TO_SAVED, NULL,
+	                plugingui_loadbutton_cb, NULL, _("_Load..."));
 
-	gtkutil_button (GTK_DIALOG (plugin_window)->action_area,
-						 GTK_STOCK_REVERT_TO_SAVED, NULL, plugingui_loadbutton_cb,
-						 NULL, _("_Load..."));
-
-	gtkutil_button (GTK_DIALOG (plugin_window)->action_area,
-						 GTK_STOCK_DELETE, NULL, plugingui_unload,
-						 NULL, _("_UnLoad"));
-
-	okb = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
-	gtk_box_pack_end (GTK_BOX (GTK_DIALOG (plugin_window)->action_area),
-							okb, 1, 1, 10);
-	g_signal_connect (G_OBJECT (okb), "clicked",
-							G_CALLBACK (plugingui_close_button), 0);
-	gtk_widget_show (okb);
+	gtkutil_button (action_area, GTK_STOCK_DELETE, NULL,
+	                plugingui_unload, NULL, _("_UnLoad"));
 
+	gtkutil_button (action_area,
+						 GTK_STOCK_CLOSE, NULL, plugingui_close_button,
+						 NULL, _("_Close"));
+ 
 	fe_pluginlist_update ();
 
 	gtk_widget_show (plugin_window);
