@@ -32,6 +32,8 @@
 #include "../common/xchatc.h"
 #include "../common/servlist.h"
 #include "../common/cfgfiles.h"
+
+#include "fe-gtk.h"
 #include "gtkutil.h"
 #include "menu.h"
 #include "pixmaps.h"
@@ -477,6 +479,7 @@ servlist_find_selected_net (GtkTreeSelection *sel, int *pos)
 	{
 		gtk_tree_model_get (model, &iter, 0, &netname, -1);
 		net = servlist_net_find (netname, pos);
+		g_free (netname);
 		if (net)
 			prefs.slist_select = *pos;
 	}
@@ -516,6 +519,7 @@ servlist_server_row_cb (GtkTreeSelection *sel, gpointer user_data)
 	{
 		gtk_tree_model_get (model, &iter, 0, &servname, -1);
 		serv = servlist_server_find (selected_net, servname, &pos);
+		g_free (servname);
 		if (serv)
 			selected_net->selected = pos;
 	}
@@ -608,11 +612,12 @@ servlist_serv_press_cb (GtkWidget *widget, GdkEventButton *event,
 	ircserver *serv;
 	GtkTreeModel *model;
 
-	if (event->type == GDK_2BUTTON_PRESS)
+	/* this conflicts with inline editing too much */
+/*	if (event->type == GDK_2BUTTON_PRESS)
 	{
 		if (selected_net != NULL)
 			servlist_connect_cb (widget, user_data);
-	}
+	}*/
 
 	if (event->button == 3)
 	{
@@ -628,6 +633,7 @@ servlist_serv_press_cb (GtkWidget *widget, GdkEventButton *event,
 			{
 				gtk_tree_model_get (model, &iter, 0, &servname, -1);
 				serv = servlist_server_find (selected_net, servname, &pos);
+				g_free (servname);
 				if (serv)
 					servlist_server_popmenu (serv, GTK_TREE_VIEW (servers_tree), event);
 			}
@@ -657,6 +663,7 @@ servlist_celledit_cb (GtkCellRendererText *cell, gchar *arg1, gchar *arg2,
 	gtk_tree_model_get (model, &iter, 0, &netname, -1);
 
 	net = servlist_net_find (netname, NULL);
+	g_free (netname);
 	if (net)
 	{
 		netname = net->name;
@@ -924,13 +931,15 @@ servlist_editserver_cb (GtkCellRendererText *cell, gchar *arg1, gchar *arg2,
 	char *servname;
 	ircserver *serv;
 
-	gtk_tree_model_get_iter (model, &iter, path);
-	gtk_tree_model_get (model, &iter, 0, &servname, -1);
-
 	if (!selected_net)
 		return;
 
+	gtk_tree_model_get_iter (model, &iter, path);
+	gtk_tree_model_get (model, &iter, 0, &servname, -1);
+
 	serv = servlist_server_find (selected_net, servname, NULL);
+	g_free (servname);
+
 	if (serv)
 	{
 		/* delete empty item */
@@ -1185,8 +1194,12 @@ fe_serverlist_open (session *sess)
 	connectnew_button = NULL;
 
 	serverlist_win = win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+	gtk_window_set_type_hint (GTK_WINDOW (win), GDK_WINDOW_TYPE_HINT_DIALOG);
+	if (current_sess)
+		gtk_window_set_transient_for (GTK_WINDOW (win), GTK_WINDOW (current_sess->gui->window));
 	gtk_window_set_title (GTK_WINDOW (win), _("X-Chat: Server List"));
 	gtk_window_set_position (GTK_WINDOW (win), GTK_WIN_POS_MOUSE);
+	gtk_window_set_role (GTK_WINDOW (win), "servlist");
 	gtkutil_set_icon (win);
 	gtk_container_set_border_width (GTK_CONTAINER (win), 5);
 
