@@ -205,8 +205,8 @@ int
 fe_userlist_remove (session *sess, struct User *user)
 {
 	GtkTreeIter *iter;
-	GtkAdjustment *adj;
-	gfloat val, end;
+/*	GtkAdjustment *adj;
+	gfloat val, end;*/
 	int sel;
 
 	iter = find_row (GTK_TREE_VIEW (sess->gui->user_tree),
@@ -214,20 +214,20 @@ fe_userlist_remove (session *sess, struct User *user)
 	if (!iter)
 		return 0;
 
-	adj = gtk_tree_view_get_vadjustment (GTK_TREE_VIEW (sess->gui->user_tree));
-	val = adj->value;
+/*	adj = gtk_tree_view_get_vadjustment (GTK_TREE_VIEW (sess->gui->user_tree));
+	val = adj->value;*/
 
 	gtk_list_store_remove (sess->res->user_model, iter);
 
 	/* is it the front-most tab? */
-	if (gtk_tree_view_get_model (GTK_TREE_VIEW (sess->gui->user_tree))
+/*	if (gtk_tree_view_get_model (GTK_TREE_VIEW (sess->gui->user_tree))
 		 == sess->res->user_model)
 	{
 		end = adj->upper - adj->lower - adj->page_size;
 		if (val > end)
 			val = end;
 		gtk_adjustment_set_value (adj, val);
-	}
+	}*/
 
 	return sel;
 }
@@ -235,26 +235,20 @@ fe_userlist_remove (session *sess, struct User *user)
 void
 fe_userlist_insert (session *sess, struct User *newuser, int row, int sel)
 {
-	char *name[2];
-	gfloat val;
 	GtkTreeModel *model = sess->res->user_model;
 	GdkPixbuf *pix;
 	GtkTreeIter iter;
 
-	name[0] = newuser->nick;
-	name[1] = newuser->hostname;
-
-	val = userlist_get_value (sess->gui->user_tree);
-
-	switch (row)
+	/* TreeView is no faster with this! Why?? */
+/*	if (after)
 	{
-	case -1:
-		gtk_list_store_append (GTK_LIST_STORE (model), &iter);
-		break;
-	default:
-		/* row 0 does an *_append() */
+		gtk_list_store_insert_after (GTK_LIST_STORE (model), iter,
+											  (GtkTreeIter *)after->gui);
+	} else
+	{*/
+		/* row 0 does an *_prepend() */
 		gtk_list_store_insert (GTK_LIST_STORE (model), &iter, row);
-	}
+	/*}*/
 
 	pix = get_user_icon (sess->server, newuser);
 
@@ -266,7 +260,7 @@ fe_userlist_insert (session *sess, struct User *newuser, int row, int sel)
 							  -1);
 
 	/* is it me? */
-	if (!sess->server->p_cmp (newuser->nick, sess->server->nick) && sess->gui->nick_box)
+	if (newuser->me && sess->gui->nick_box)
 	{
 		sess->res->myself = newuser;
 		if (!sess->gui->is_tab || sess == current_tab)
@@ -283,10 +277,8 @@ fe_userlist_insert (session *sess, struct User *newuser, int row, int sel)
 
 	/* is it the front-most tab? */
 	if (gtk_tree_view_get_model (GTK_TREE_VIEW (sess->gui->user_tree))
-		 == sess->res->user_model)
+		 == model)
 	{
-		userlist_set_value (sess->gui->user_tree, val);
-
 		if (sel)
 			gtk_tree_selection_select_iter (gtk_tree_view_get_selection
 										(GTK_TREE_VIEW (sess->gui->user_tree)), &iter);
@@ -391,16 +383,12 @@ userlist_add_columns (GtkTreeView * treeview)
 	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
 																-1, NULL, renderer,
 																"pixbuf", 0, NULL);
-	gtk_tree_view_column_set_sizing (gtk_tree_view_get_column (treeview, 0),
-												GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 
 	/* nick column */
 	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
 																-1, NULL, renderer,
 																"text", 1, NULL);
-	gtk_tree_view_column_set_sizing (gtk_tree_view_get_column (treeview, 1),
-												GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 
 	if (prefs.showhostname_in_userlist)
 	{
@@ -430,6 +418,11 @@ userlist_click_cb (GtkWidget *widget, GdkEventButton *event, gpointer userdata)
 		{
 			nick_command_parse (current_sess, prefs.doubleclickuser, nicks[0],
 									  nicks[0]);
+			while (i)
+			{
+				i--;
+				g_free (nicks[i]);
+			}
 			free (nicks);
 		}
 		return TRUE;
@@ -442,6 +435,11 @@ userlist_click_cb (GtkWidget *widget, GdkEventButton *event, gpointer userdata)
 		if (nicks && i > 1)
 		{
 			menu_nickmenu (current_sess, event, nicks[0], i);
+			while (i)
+			{
+				i--;
+				g_free (nicks[i]);
+			}
 			free (nicks);
 			return TRUE;
 		}
@@ -459,6 +457,11 @@ userlist_click_cb (GtkWidget *widget, GdkEventButton *event, gpointer userdata)
 			if (nicks)
 			{
 				menu_nickmenu (current_sess, event, nicks[0], i);
+				while (i)
+				{
+					i--;
+					g_free (nicks[i]);
+				}
 				free (nicks);
 			}
 		} else
